#include <stdio.h>
#include <string.h>
#ifdef __W32__
  #include <windows.h>
  #define sleep( a ) Sleep( a * 1000 )
#endif
#include <ctype.h>
#include <stdarg.h>

#ifndef HZ
  #define HZ 100
#endif

#ifndef max
   #define max(a, b) (((a) > (b)) ? (a) : (b))
#endif
#ifndef min
   #define min(a, b) (((a) < (b)) ? (a) : (b))
#endif

// CPU: [===---    ]   MEM:   9.1MB   LAN:  13.4kB/s(rcv) 634.0 B/s(snd) 

char* scale( char* pcBuffer, unsigned long ul );
char* graph( char* pcBuffer, unsigned long ulSys, unsigned long ulNic, unsigned long ulUsr );
int   rdval( char* pcFileName, char* pcKey, unsigned long *pulItem, ... );

//============================================================================================================
int main( int argc, char* argv[] ) {
//============================================================================================================

    char sCPU[ 11 ];
    char sMEM[ 8 ];
    char sRCV[ 8 ], sSND[ 8 ];

    unsigned long ulMEM[ 1 ] = { 0 };
    unsigned long ulCPU[ 4 ] = { 0, 0, 0, 0 };
    unsigned long ulLAN[ 4 ] = { 0, 0, 0, 0 };

    unsigned long ulCPUOld[ 4 ] = { 0, 0, 0, 0 };
    unsigned long ulLANOld[ 4 ] = { 0, 0, 0, 0 };  // [1] and [3] not used.

    char *device     = argc > 1 ? argv[ 1 ] : "eth0:";
    char controlchar = argc > 2 ? '\r' : '\n';

    putchar( '\n' );

    do {

        unsigned long ulWork;
        unsigned long ulTime = 0;
        int i;

        ulLAN[ 0 ] = ulLAN[ 1 ] = 0;
        rdval( "/proc/stat",    "cpu",  ulCPU, 1, 2, 3, 4 );
        rdval( "/proc/meminfo", "Mem:", ulMEM, 2 );
        rdval( "/proc/net/dev", device, ulLAN, 1, 3, 9, 11 );

        for ( i = 0; i < 4; i++ ) {
            ulWork = ulCPU[ i ] - ulCPUOld[ i ];
            ulTime += ulWork;
            ulCPUOld[ i ] = ulCPU[ i ];
            ulCPU[ i ] = ulWork;
        } /* endfor */

        for ( i = 0; i < 4; i += 2 ) {
            ulWork = ulLAN[ i ] - ulLANOld[ i ];
            ulLANOld[ i ] = ulLAN[ i ];
            ulLAN[ i ] = ulWork;
        } /* endfor */

        if ( ulTime != 0 ) {
            printf( "%cCPU: [%s]   MEM: %sB   LAN: %sB/s(rcv) %sB/s(snd)  %lu(err)"
                  , controlchar
                  , graph( sCPU, 100 * ulCPU[ 2 ] / ulTime, 100 * ulCPU[ 1 ] / ulTime, 100 * ulCPU[ 0 ] / ulTime )
                  , scale( sMEM, ulMEM[ 0 ] )
                  , scale( sRCV, HZ * ulLAN[ 0 ] / ulTime )
                  , scale( sSND, HZ * ulLAN[ 2 ] / ulTime )
                  , ulLAN[ 1 ] + ulLAN[ 3 ] );
            fflush( stdout );
        } /* endif */
        sleep( 1 );
    } while ( 1 );

    return 0;
}

//------------------------------------------------------------------------------------------------------------
char* scale( char* pcBuffer, unsigned long ul ) {
//------------------------------------------------------------------------------------------------------------

    int index = 0;
    ul *= 10;
    while ( ul >= 10000 ) {
        ul /= 1024;
        index++;
    } /* endwhile */

    sprintf( pcBuffer
           , "%3d.%d%c"
           , ul / 10
           , ul % 10
           , "0kMGTEP"[ index ] );
    return pcBuffer;
}

//------------------------------------------------------------------------------------------------------------
char* graph( char* pcBuffer, unsigned long ulSys, unsigned long ulNic, unsigned long ulUsr ) {
//------------------------------------------------------------------------------------------------------------

    unsigned long ulFre = 10;
    ulSys = min( ulFre, ( ulSys + 9 ) / 10 );
    ulFre -= ulSys;
    ulNic = min( ulFre, ( ulNic + 9 ) / 10 );
    ulFre -= ulNic;
    ulUsr = min( ulFre, ( ulUsr + 9 ) / 10 );
    ulFre -= ulUsr;

    sprintf( pcBuffer
           , "%.*s%.*s%.*s%.*s"
           , ulSys, "=========="
           , ulNic, "++++++++++"
           , ulUsr, "----------"
           , ulFre, "          " );   
    return pcBuffer;
}

//------------------------------------------------------------------------------------------------------------
int rdval( char* pcFileName, char* pcKey, unsigned long *pulItem, ... ) {
//------------------------------------------------------------------------------------------------------------

    FILE* pFile;
    char s[ 256 ];
    char *p;

    if (( pFile = fopen( pcFileName, "r" )) != NULL ) {
        while ( fgets( s, sizeof( s ), pFile ) != NULL ) {
            p = s;
            while ( *p != '\0' && isspace( *p )) {
                p++;
            } /* endwhile */
            if ( strncmp( p, pcKey, strlen( pcKey )) == 0 ) {

                va_list arg_ptr;
                int indexline = 1;
                int indexdata = 0;
                int indexnext;

                p += strlen( pcKey );
                while ( *p != '\0' && isspace( *p )) {
                    p++;
                } /* endwhile */

                va_start( arg_ptr, pulItem );
                indexnext = va_arg( arg_ptr, int );
                strtok( p, " \t" );
                do {
                    if ( indexline == indexnext ) {
                        pulItem[ indexdata ] = strtoul( p, NULL, 10 );
                        indexnext = va_arg( arg_ptr, int );
                        indexdata++;
                    } /* endif */
                    indexline++;
                } while (( p = strtok( NULL, " \t" )) != NULL ); /* endwhile */
                va_end( arg_ptr );

                fclose( pFile );
                return 0;

            } /* endif */
        } /* endwhile */
        fclose( pFile );
    } /* endif */

    return 1;
}
